@@ -21,6 +21,10 @@
 #include <libintl.h>
 #endif
 
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+
 
 
 gint
@@ -30,6 +34,7 @@ main (gint argc, gchar **argv)
   gint                 status;
 
   /* bind the text domain to the locale directory */
+  setlocale (LC_ALL, "");
   bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
