@@ -15,16 +15,11 @@
  */
 
 #include <mousepad/mousepad-private.h>
-#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-dialogs.h>
 #include <mousepad/mousepad-util.h>
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 #include <gtksourceview/gtksource.h>
-#else
-#include <gtksourceview/gtksourcestylescheme.h>
-#include <gtksourceview/gtksourcestyleschememanager.h>
-#endif
+
 
 
 void
@@ -35,6 +30,7 @@ mousepad_dialogs_show_about (GtkWindow *parent)
     "Nick Schermer <nick@xfce.org>",
     "Erik Harrison <erikharrison@xfce.org>",
     "Matthew Brush <matt@xfce.org>",
+    "Gaël Bonithon <gael@xfce.org>",
     NULL
   };
 
@@ -42,12 +38,14 @@ mousepad_dialogs_show_about (GtkWindow *parent)
   gtk_show_about_dialog (parent,
                          "authors", authors,
                          "comments", _("Mousepad is a fast text editor for the Xfce Desktop Environment."),
+                         "copyright", "Copyright \xc2\xa9 2005-2020 - the Mousepad developers",
                          "destroy-with-parent", TRUE,
-                         "logo-icon-name", "accessories-text-editor",
+                         "license-type", GTK_LICENSE_GPL_2_0,
+                         "logo-icon-name", "org.xfce.mousepad",
                          "program-name", PACKAGE_NAME,
                          "version", PACKAGE_VERSION,
                          "translator-credits", _("translator-credits"),
-                         "website", "http://www.xfce.org/",
+                         "website", "https://docs.xfce.org/apps/mousepad/start",
                          NULL);
 }
 
@@ -85,9 +83,23 @@ mousepad_dialogs_show_help (GtkWindow   *parent,
                             const gchar *page,
                             const gchar *offset)
 {
+#if !GTK_CHECK_VERSION (3, 22, 0)
   GdkScreen   *screen;
+#endif
   GError      *error = NULL;
-  const gchar *uri;
+  const gchar *uri = "https://docs.xfce.org/apps/mousepad/start";
+
+  /* try to run the documentation browser */
+#if GTK_CHECK_VERSION (3, 22, 0)
+  if (!gtk_show_uri_on_window (parent, uri, gtk_get_current_event_time (), &error))
+    {
+      /* display an error message to the user */
+      mousepad_dialogs_show_error (parent, error, _("Failed to open the documentation browser"));
+      g_error_free (error);
+    }
+#else
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 
   /* get screen */
   if (G_LIKELY (parent))
@@ -95,15 +107,16 @@ mousepad_dialogs_show_help (GtkWindow   *parent,
   else
     screen = gdk_screen_get_default ();
 
-  uri = "http://docs.xfce.org";
-
-  /* try to run the documentation browser */
   if (!gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error))
     {
       /* display an error message to the user */
       mousepad_dialogs_show_error (parent, error, _("Failed to open the documentation browser"));
       g_error_free (error);
     }
+
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+#endif
 }
 
 
@@ -113,14 +126,15 @@ mousepad_dialogs_other_tab_size (GtkWindow *parent,
                                  gint      active_size)
 {
   GtkWidget *dialog;
+  GtkWidget *area;
   GtkWidget *scale;
 
   /* build dialog */
   dialog = gtk_dialog_new_with_buttons (_("Select Tab Size"),
                                         parent,
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL,
-                                        GTK_STOCK_OK, MOUSEPAD_RESPONSE_OK,
+                                        _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL,
+                                        _("_OK"), MOUSEPAD_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_OK);
 
@@ -130,7 +144,8 @@ mousepad_dialogs_other_tab_size (GtkWindow *parent,
   gtk_scale_set_digits (GTK_SCALE (scale), 0);
   gtk_scale_set_draw_value (GTK_SCALE (scale), TRUE);
   gtk_scale_set_value_pos (GTK_SCALE (scale), GTK_POS_TOP);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG(dialog))), scale, TRUE, TRUE, 0);
+  area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_pack_start (GTK_BOX (area), scale, TRUE, TRUE, 0);
   gtk_widget_show (scale);
 
   /* run the dialog */
@@ -179,11 +194,10 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
                         GtkTextBuffer *buffer)
 {
   GtkWidget    *dialog;
-  GtkWidget    *vbox;
-  GtkWidget    *hbox;
+  GtkWidget    *area, *vbox, *hbox;
+  GtkWidget    *button;
   GtkWidget    *label;
-  GtkWidget    *line_spin;
-  GtkWidget    *col_spin;
+  GtkWidget    *line_spin, *col_spin;
   GtkSizeGroup *size_group;
   GtkTextIter   iter;
   gint          line, column, lines;
@@ -200,14 +214,17 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
   dialog = gtk_dialog_new_with_buttons (_("Go To"),
                                         parent,
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL,
-                                        GTK_STOCK_JUMP_TO, MOUSEPAD_RESPONSE_JUMP_TO,
+                                        _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL,
                                         NULL);
+  button = mousepad_util_image_button ("go-jump", _("_Jump to"));
+  gtk_widget_set_can_default (button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_JUMP_TO);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_JUMP_TO);
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 
   vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG(dialog))), vbox, TRUE, TRUE, 0);
+  area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  gtk_box_pack_start (GTK_BOX (area), vbox, TRUE, TRUE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_widget_show (vbox);
 
@@ -222,7 +239,8 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
   label = gtk_label_new_with_mnemonic (_("_Line number:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
   gtk_size_group_add_widget (size_group, label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
   gtk_widget_show (label);
 
   line_spin = gtk_spin_button_new_with_range (1, lines, 1);
@@ -241,7 +259,8 @@ mousepad_dialogs_go_to (GtkWindow     *parent,
   label = gtk_label_new_with_mnemonic (_("C_olumn number:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
   gtk_size_group_add_widget (size_group, label);
-  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
   gtk_widget_show (label);
 
   col_spin = gtk_spin_button_new_with_range (0, 0, 1);
@@ -290,28 +309,53 @@ gboolean
 mousepad_dialogs_clear_recent (GtkWindow *parent)
 {
   GtkWidget *dialog;
+  GtkWidget *area, *hbox, *vbox;
+  GtkWidget *button;
   GtkWidget *image;
+  GtkWidget *label;
   gboolean   succeed = FALSE;
 
   /* create the question dialog */
-  dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_OTHER, GTK_BUTTONS_NONE,
-                                   _("Remove all entries from the documents history?"));
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL,
-                          GTK_STOCK_CLEAR, MOUSEPAD_RESPONSE_CLEAR,
-                          NULL);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Clear Documents History"));
+  dialog = gtk_dialog_new_with_buttons (_("Clear Documents History"),
+                                        parent,
+                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL,
+                                        NULL);
+  button = mousepad_util_image_button ("edit-clear", _("Clea_r"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_CLEAR);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_CANCEL);
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                            _("Clearing the documents history will permanently "
-                                              "remove all currently listed entries."));
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 400, -1);
+
+  /* the content area */
+  area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_box_pack_start (GTK_BOX (area), hbox, TRUE, TRUE, 6);
+  gtk_widget_show (hbox);
 
   /* the dialog icon */
-  image = gtk_image_new_from_stock (GTK_STOCK_CLEAR, GTK_ICON_SIZE_DIALOG);
-  gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+  image = gtk_image_new_from_icon_name ("edit-clear", GTK_ICON_SIZE_DIALOG);
+  gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
   gtk_widget_show (image);
 
+  /* the dialog message */
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 6);
+  gtk_widget_show (vbox);
+
+  /* primary text */
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label),
+                        _("<big><b>Remove all entries from the documents history?</b></big>"));
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show (label);
+
+  /* secondary text */
+  label = gtk_label_new (_("Clearing the documents history will permanently "
+                           "remove all currently listed entries."));
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show (label);
+
   /* popup the dialog */
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == MOUSEPAD_RESPONSE_CLEAR)
     succeed = TRUE;
@@ -329,37 +373,67 @@ mousepad_dialogs_save_changes (GtkWindow *parent,
                                gboolean   readonly)
 {
   GtkWidget *dialog;
+  GtkWidget *area, *hbox, *vbox;
+  GtkWidget *button;
   GtkWidget *image;
+  GtkWidget *label;
   gint       response;
 
   /* create the question dialog */
-  dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_OTHER, GTK_BUTTONS_NONE,
-                                   _("Do you want to save the changes before closing?"));
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Save Changes"));
-  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), mousepad_util_image_button (GTK_STOCK_DELETE, _("_Don't Save")), MOUSEPAD_RESPONSE_DONT_SAVE);
-  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL);
+  dialog = gtk_dialog_new_with_buttons (_("Save Changes"),
+                                        parent,
+                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL,
+                                        NULL);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 400, -1);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
+                                mousepad_util_image_button ("edit-delete", _("_Don't Save")),
+                                MOUSEPAD_RESPONSE_DONT_SAVE);
 
   /* we show the save as button instead of save for readonly document */
   if (G_UNLIKELY (readonly))
     {
-      image = gtk_image_new_from_stock (GTK_STOCK_SAVE_AS, GTK_ICON_SIZE_DIALOG);
-      gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_SAVE_AS, MOUSEPAD_RESPONSE_SAVE_AS, NULL);
+      image = gtk_image_new_from_icon_name ("document-save-as", GTK_ICON_SIZE_DIALOG);
+      button = mousepad_util_image_button ("document-save-as", _("_Save As"));
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_SAVE_AS);
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_SAVE_AS);
     }
   else
     {
-      image = gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_DIALOG);
-      gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_SAVE, MOUSEPAD_RESPONSE_SAVE, NULL);
+      image = gtk_image_new_from_icon_name ("document-save", GTK_ICON_SIZE_DIALOG);
+      button = mousepad_util_image_button ("document-save", _("_Save"));
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_SAVE);
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_SAVE);
     }
 
+  /* the content area */
+  area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
+  gtk_box_pack_start (GTK_BOX (area), hbox, TRUE, TRUE, 6);
+  gtk_widget_show (hbox);
+
   /* the dialog icon */
-  gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+  gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
   gtk_widget_show (image);
 
+  /* the dialog message */
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 8);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 6);
+  gtk_widget_show (vbox);
+
+  /* primary text */
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label),
+                        _("<big><b>Do you want to save the changes before closing?</b></big>"));
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show (label);
+
   /* secondary text */
-  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), _("If you don't save the document, all the changes will be lost."));
+  label = gtk_label_new (_("If you don't save the document, all the changes will be lost."));
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+  gtk_widget_show (label);
 
   /* run the dialog and wait for a response */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -376,19 +450,23 @@ gint
 mousepad_dialogs_externally_modified (GtkWindow *parent)
 {
   GtkWidget *dialog;
+  GtkWidget *button;
   gint       response;
 
   /* create the question dialog */
   dialog = gtk_message_dialog_new (parent, GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_WARNING, GTK_BUTTONS_NONE,
-                                   _("The document has been externally modified. Do you want to continue saving?"));
+                                   _("The document has been externally modified. "
+                                     "Do you want to continue saving?"));
   gtk_window_set_title (GTK_WINDOW (dialog), _("Externally Modified"));
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                            _("If you save the document, all of the external changes will be lost."));
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL,
-                          GTK_STOCK_SAVE_AS, MOUSEPAD_RESPONSE_SAVE_AS,
-                          GTK_STOCK_SAVE, MOUSEPAD_RESPONSE_SAVE, NULL);
+                                            _("If you save the document, all of the external "
+                                              "changes will be lost."));
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL, NULL);
+  button = mousepad_util_image_button ("document-save-as", _("Save _As"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_SAVE_AS);
+  button = mousepad_util_image_button ("document-save", _("_Save"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_SAVE);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), MOUSEPAD_RESPONSE_CANCEL);
 
   /* run the dialog */
@@ -406,6 +484,7 @@ gint
 mousepad_dialogs_revert (GtkWindow *parent)
 {
   GtkWidget *dialog;
+  GtkWidget *button;
   gint       response;
 
   /* setup the question dialog */
@@ -415,11 +494,11 @@ mousepad_dialogs_revert (GtkWindow *parent)
                                    _("Do you want to save your changes before reloading?"));
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                    _("If you revert the file, all unsaved changes will be lost."));
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          GTK_STOCK_CANCEL, MOUSEPAD_RESPONSE_CANCEL,
-                          GTK_STOCK_SAVE_AS, MOUSEPAD_RESPONSE_SAVE_AS,
-                          GTK_STOCK_REVERT_TO_SAVED, MOUSEPAD_RESPONSE_REVERT,
-                          NULL);
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Cancel"), MOUSEPAD_RESPONSE_CANCEL, NULL);
+  button = mousepad_util_image_button ("document-save-as", _("_Save As"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_SAVE_AS);
+  button = mousepad_util_image_button ("document-revert", _("_Revert"));
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, MOUSEPAD_RESPONSE_REVERT);
 
   /* run the dialog */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -429,3 +508,101 @@ mousepad_dialogs_revert (GtkWindow *parent)
 
   return response;
 }
+
+
+
+gint
+mousepad_dialogs_save_as (GtkWindow    *parent,
+                          const gchar  *current_filename,
+                          const gchar  *last_save_location,
+                          gchar       **filename)
+{
+  GtkWidget *dialog, *button;
+  gint       response;
+
+  /* create the dialog */
+  dialog = gtk_file_chooser_dialog_new (_("Save As"),
+                                        parent, GTK_FILE_CHOOSER_ACTION_SAVE,
+                                        _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                        NULL);
+
+  button = mousepad_util_image_button ("document-save", _("_Save"));
+  gtk_widget_set_can_default (button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
+  gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), TRUE);
+  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+  /* set the current filename if there is one, or use the last save location */
+  if (current_filename)
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), current_filename);
+  else if (last_save_location)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), last_save_location);
+
+  /* run the dialog */
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  /* get the new filename */
+  *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+
+  /* destroy the dialog */
+  gtk_widget_destroy (dialog);
+
+  return response;
+}
+
+
+
+gint
+mousepad_dialogs_open (GtkWindow    *parent,
+                       const gchar  *filename,
+                       GSList      **filenames)
+{
+  GtkWidget *dialog, *button, *hbox, *label, *combobox;
+  gint       response;
+
+  /* create new file chooser dialog */
+  dialog = gtk_file_chooser_dialog_new (_("Open File"),
+                                         parent,
+                                         GTK_FILE_CHOOSER_ACTION_OPEN,
+                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                         NULL);
+  button = mousepad_util_image_button ("document-open", _("_Open"));
+  gtk_widget_set_can_default (button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_ACCEPT);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+  gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), TRUE);
+  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), TRUE);
+
+  /* encoding selector */
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), hbox);
+  gtk_widget_show (hbox);
+
+  label = gtk_label_new_with_mnemonic (_("_Encoding:"));
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0);
+  gtk_label_set_yalign (GTK_LABEL (label), 0.5);
+  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+  gtk_widget_show (label);
+
+  combobox = gtk_combo_box_new ();
+  gtk_box_pack_start (GTK_BOX (hbox), combobox, FALSE, FALSE, 0);
+  gtk_label_set_mnemonic_widget (GTK_LABEL (label), combobox);
+  gtk_widget_show (combobox);
+
+  /* select the active document in the file chooser */
+  if (filename && g_file_test (filename, G_FILE_TEST_EXISTS))
+    gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog), filename);
+
+  /* run the dialog */
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  /* get a list of selected filenames */
+  *filenames = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dialog));
+
+  /* destroy the dialog */
+  gtk_widget_destroy (dialog);
+
+  return response;
+}
