@@ -15,18 +15,14 @@
  */
 
 #include <mousepad/mousepad-private.h>
-#include <mousepad/mousepad-gtkcompat.h>
 #include <mousepad/mousepad-file.h>
+#if !GLIB_CHECK_VERSION (2, 52, 0)
+#include <mousepad/mousepad-util.h>
+#endif
 
 #include <glib/gstdio.h>
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 #include <gtksourceview/gtksource.h>
-#else
-#include <gtksourceview/gtksourcebuffer.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagemanager.h>
-#endif
 
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -44,6 +40,7 @@
 #include <errno.h>
 
 
+
 enum
 {
   /* EXTERNALLY_MODIFIED, */
@@ -66,6 +63,7 @@ struct _MousepadFile
 
   /* filename */
   gchar              *filename;
+  gboolean            temporary;
 
   /* encoding of the file */
   MousepadEncoding    encoding;
@@ -88,9 +86,9 @@ struct _MousepadFile
 
 
 
-static void  mousepad_file_finalize         (GObject            *object);
-static void  mousepad_file_set_readonly     (MousepadFile       *file,
-                                             gboolean            readonly);
+static void  mousepad_file_finalize          (GObject            *object);
+static void  mousepad_file_set_read_only     (MousepadFile       *file,
+                                              gboolean            readonly);
 
 
 
@@ -145,6 +143,7 @@ mousepad_file_init (MousepadFile *file)
 {
   /* initialize */
   file->filename          = NULL;
+  file->temporary         = FALSE;
   file->encoding          = MOUSEPAD_ENCODING_UTF_8;
 #ifdef G_OS_WIN32
   file->line_ending       = MOUSEPAD_EOL_DOS;
@@ -175,24 +174,6 @@ mousepad_file_finalize (GObject *object)
 
 
 
-static void
-mousepad_file_set_readonly (MousepadFile *file,
-                            gboolean      readonly)
-{
-  g_return_if_fail (MOUSEPAD_IS_FILE (file));
-
-  if (G_LIKELY (file->readonly != readonly))
-    {
-      /* store new value */
-      file->readonly = readonly;
-
-      /* emit signal */
-      g_signal_emit (G_OBJECT (file), file_signals[READONLY_CHANGED], 0, readonly);
-    }
-}
-
-
-
 static MousepadEncoding
 mousepad_file_encoding_read_bom (const gchar *contents,
                                  gsize        length,
@@ -281,22 +262,32 @@ mousepad_file_new (GtkTextBuffer *buffer)
 
 void
 mousepad_file_set_filename (MousepadFile *file,
-                            const gchar  *filename)
+                            const gchar  *filename,
+                            gboolean      real)
 {
   g_return_if_fail (MOUSEPAD_IS_FILE (file));
 
-  /* reset the stored modification time when a new filename set */
   if (g_strcmp0 (file->filename, filename) != 0)
-    file->mtime = 0;
+    {
+      /* free the old filename */
+      g_free (file->filename);
 
-  /* free the old filename */
-  g_free (file->filename);
+      /* set the filename */
+      file->filename = g_strdup (filename);
+    }
 
-  /* set the filename */
-  file->filename = g_strdup (filename);
+  /* not a virtual change, such as when trying to save as */
+  if (real)
+    {
+      /* this is a definitve filename */
+      file->temporary = FALSE;
 
-  /* send a signal that the name has been changed */
-  g_signal_emit (G_OBJECT (file), file_signals[FILENAME_CHANGED], 0, file->filename);
+      /* send a signal that the name has been changed */
+      g_signal_emit (G_OBJECT (file), file_signals[FILENAME_CHANGED], 0, file->filename);
+    }
+  /* toggle filename state */
+  else
+    file->temporary = ! file->temporary;
 }
 
 
@@ -343,6 +334,24 @@ mousepad_file_get_encoding (MousepadFile *file)
 
 
 
+static void
+mousepad_file_set_read_only (MousepadFile *file,
+                             gboolean      readonly)
+{
+  g_return_if_fail (MOUSEPAD_IS_FILE (file));
+
+  if (G_LIKELY (file->readonly != readonly))
+    {
+      /* store new value */
+      file->readonly = readonly;
+
+      /* emit signal */
+      g_signal_emit (G_OBJECT (file), file_signals[READONLY_CHANGED], 0, readonly);
+    }
+}
+
+
+
 gboolean
 mousepad_file_get_read_only (MousepadFile *file)
 {
@@ -462,15 +471,24 @@ mousepad_file_guess_language (MousepadFile *file)
   gchar             *basename;
   gboolean           result_uncertain;
   GtkSourceLanguage *language;
+  gchar             *data;
+  GtkTextIter        start;
+  GtkTextIter        end;
 
-  g_return_val_if_fail ((file->filename != NULL), NULL);
+  g_return_val_if_fail (file->filename != NULL, NULL);
 
-  content_type = g_content_type_guess (file->filename, NULL, 0, &result_uncertain);
+  gtk_text_buffer_get_start_iter (file->buffer, &start);
+  end = start;
+  gtk_text_iter_forward_chars (&end, 255);
+  data = gtk_text_buffer_get_text (file->buffer, &start, &end, TRUE);
+
+  content_type = g_content_type_guess (file->filename, (const guchar *) data, strlen (data), &result_uncertain);
   basename = g_path_get_basename (file->filename);
   language = gtk_source_language_manager_guess_language (gtk_source_language_manager_get_default (),
                                                          basename,
                                                          result_uncertain ? NULL : content_type);
 
+  g_free (data);
   g_free (basename);
   g_free (content_type);
 
@@ -485,17 +503,13 @@ mousepad_file_open (MousepadFile  *file,
                     GError       **error)
 {
   GMappedFile      *mapped_file;
-  const gchar      *filename;
-  gint              retval = ERROR_READING_FAILED;
-  gsize             file_size, written;
-  gsize             bom_length;
-  const gchar      *contents;
-  gchar            *encoded = NULL;
-  const gchar      *charset;
+  MousepadEncoding  bom_encoding;
   GtkTextIter       start_iter, end_iter;
   struct stat       statb;
-  const gchar      *end, *n, *m;
-  MousepadEncoding  bom_encoding;
+  gint              retval = ERROR_READING_FAILED;
+  gsize             file_size, written, bom_length;
+  const gchar      *filename, *contents, *charset, *end, *n, *m;
+  gchar            *encoded = NULL, *utf8_valid = NULL;;
 
   g_return_val_if_fail (MOUSEPAD_IS_FILE (file), FALSE);
   g_return_val_if_fail (GTK_IS_TEXT_BUFFER (file->buffer), FALSE);
@@ -512,7 +526,7 @@ mousepad_file_open (MousepadFile  *file,
   if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
     {
       /* update readonly status */
-      mousepad_file_set_readonly (file, FALSE);
+      mousepad_file_set_read_only (file, FALSE);
 
       return 0;
     }
@@ -543,47 +557,47 @@ mousepad_file_open (MousepadFile  *file,
               file->encoding = bom_encoding;
             }
 
-          /* handle encoding and check for utf-8 valid text */
-          if (G_LIKELY (file->encoding == MOUSEPAD_ENCODING_UTF_8))
+          /* leave when the contents is not utf-8 valid */
+          if (G_LIKELY (file->encoding == MOUSEPAD_ENCODING_UTF_8)
+              && g_utf8_validate_len (contents, file_size, &end) == FALSE)
             {
-              validate:
+              /* set return value */
+              retval = ERROR_NOT_UTF8_VALID;
 
-              /* leave when the contents is not utf-8 valid */
-              if (g_utf8_validate (contents, file_size, &end) == FALSE)
-                {
-                  /* set return value */
-                  retval = ERROR_NOT_UTF8_VALID;
+              /* set an error */
+              g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
+                           _("Invalid byte sequence in conversion input"));
 
-                  /* set an error */
-                  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
-                               _("Invalid byte sequence in conversion input"));
-
-                  goto failed;
-                }
+              goto failed;
             }
           else
             {
-              /* get the encoding charset */
-              charset = mousepad_encoding_get_charset (file->encoding);
-
-              /* convert the contents */
-              encoded = g_convert (contents, file_size, "UTF-8", charset, NULL, &written, error);
-
-              /* check if the string is utf-8 valid */
-              if (G_UNLIKELY (encoded == NULL))
+              if (file->encoding != MOUSEPAD_ENCODING_UTF_8_FORCED)
                 {
-                  /* set return value */
-                  retval = ERROR_CONVERTING_FAILED;
+                  /* get the encoding charset */
+                  charset = mousepad_encoding_get_charset (file->encoding);
 
-                  goto failed;
+                  /* convert the contents */
+                  encoded = g_convert (contents, file_size, "UTF-8", charset, NULL, &written, error);
+
+                  /* check if the encoding succeed */
+                  if (G_UNLIKELY (encoded == NULL))
+                    {
+                      /* set return value */
+                      retval = ERROR_CONVERTING_FAILED;
+
+                      goto failed;
+                    }
+
+                  /* set new values */
+                  contents = encoded;
+                  file_size = written;
                 }
 
-              /* set new values */
-              contents = encoded;
-              file_size = written;
-
-              /* validate the converted content */
-              goto validate;
+              /* force UTF-8 encoding validity and update location for end of valid data */
+              utf8_valid = g_utf8_make_valid (contents, file_size);
+              contents = utf8_valid;
+              g_utf8_validate (contents, -1, &end);
             }
 
           /* detect the line ending, based on the first eol we match */
@@ -609,8 +623,9 @@ mousepad_file_open (MousepadFile  *file,
             }
 
           /* text view doesn't expect a line ending at end of last line, but Unix and Mac files do */
-          if ((n = g_utf8_find_prev_char(contents, end)) && (*n == '\r' ||
-              (*n == '\n' && (!(n = g_utf8_find_prev_char(contents, n)) || *n != '\r'))))
+          if ((n = g_utf8_find_prev_char (contents, end))
+              && (*n == '\r' || (*n == '\n' && (! (n = g_utf8_find_prev_char (contents, n))
+                                                || *n != '\r'))))
             end--;
 
           /* get the iter at the beginning of the document */
@@ -654,7 +669,7 @@ mousepad_file_open (MousepadFile  *file,
           if (G_LIKELY (g_stat (file->filename, &statb) == 0))
             {
               /* whether the file is readonly (ie. not writable by the user) */
-              mousepad_file_set_readonly (file, !((statb.st_mode & S_IWUSR) != 0));
+              mousepad_file_set_read_only (file, !((statb.st_mode & S_IWUSR) != 0));
 
               /* store the file modification time */
               file->mtime = statb.st_mtime;
@@ -669,7 +684,7 @@ mousepad_file_open (MousepadFile  *file,
         {
           /* this is a new document with content from a template */
           file->mtime = 0;
-          mousepad_file_set_readonly (file, FALSE);
+          mousepad_file_set_read_only (file, FALSE);
         }
 
       failed:
@@ -683,13 +698,10 @@ mousepad_file_open (MousepadFile  *file,
 
       /* cleanup */
       g_free (encoded);
+      g_free (utf8_valid);
 
       /* close the mapped file */
-#if GLIB_CHECK_VERSION (2, 21, 0)
       g_mapped_file_unref (mapped_file);
-#else
-      g_mapped_file_free (mapped_file);
-#endif
 
       if (G_LIKELY (filename != template_filename))
         {
@@ -774,7 +786,7 @@ mousepad_file_save (MousepadFile  *file,
               contents = g_realloc (contents, length + 5);
 
               /* move the existing contents 3 bytes */
-              g_memmove (contents + 3, contents, length + 1);
+              memmove (contents + 3, contents, length + 1);
 
               /* write an utf-8 bom at the start of the contents */
               contents[0] = (guchar) 0xef;
@@ -853,7 +865,7 @@ mousepad_file_save (MousepadFile  *file,
           gtk_text_buffer_set_modified (file->buffer, FALSE);
 
           /* we saved succesfully */
-          mousepad_file_set_readonly (file, FALSE);
+          mousepad_file_set_read_only (file, FALSE);
 
           /* if the user hasn't set the filetype, try and re-guess it now
            * that we have a new filename to go by */
@@ -930,7 +942,7 @@ mousepad_file_get_externally_modified (MousepadFile  *file,
 
   /* check if our modification time differs from the current one */
   if (G_LIKELY (g_stat (file->filename, &statb) == 0))
-    return (file->mtime > 0 && statb.st_mtime != file->mtime);
+    return (! file->temporary && file->mtime > 0 && statb.st_mtime != file->mtime);
 
   /* get the error code */
   error_code = g_file_error_from_errno (errno);
