@@ -16,15 +16,10 @@
 
 #include <mousepad/mousepad-private.h>
 #include <mousepad/mousepad-prefs-dialog.h>
-#include <mousepad/mousepad-prefs-dialog-ui.h>
 #include <mousepad/mousepad-settings.h>
+#include <mousepad/mousepad-util.h>
 
-#if GTK_CHECK_VERSION(3, 0, 0)
 #include <gtksourceview/gtksource.h>
-#else
-#include <gtksourceview/gtksourcestylescheme.h>
-#include <gtksourceview/gtksourcestyleschememanager.h>
-#endif
 
 
 
@@ -59,6 +54,7 @@
 #define WID_REMEMBER_STATE_CHECK            "/prefs/window/general/remember-window-state-check"
 #define WID_ALWAYS_SHOW_TABS_CHECK          "/prefs/window/notebook/always-show-tabs-check"
 #define WID_CYCLE_TABS_CHECK                "/prefs/window/notebook/cycle-tabs-check"
+#define WID_OPENING_MODE_COMBO              "/prefs/window/notebook/opening-mode-combo"
 #define WID_TOOLBAR_VISIBLE_CHECK           "/prefs/window/toolbar/visible-check"
 #define WID_TOOLBAR_STYLE_COMBO             "/prefs/window/toolbar/style-combo"
 #define WID_TOOLBAR_STYLE_LABEL             "/prefs/window/toolbar/style-label"
@@ -382,7 +378,7 @@ mousepad_prefs_dialog_toolbar_icon_size_setting_changed (MousepadPrefsDialog *se
   combo = GTK_COMBO_BOX (gtk_builder_get_object (self->builder, WID_TOOLBAR_ICON_SIZE_COMBO));
   model = gtk_combo_box_get_model (combo);
   valid = gtk_tree_model_get_iter_first (model, &iter);
-  
+
   while (valid)
     {
       gint size = 0;
@@ -398,6 +394,37 @@ mousepad_prefs_dialog_toolbar_icon_size_setting_changed (MousepadPrefsDialog *se
 
 
 
+/* update opening mode setting when combo changes */
+static void
+mousepad_prefs_dialog_opening_mode_changed (MousepadPrefsDialog *self,
+                                            GtkComboBox         *combo)
+{
+  self->blocked = TRUE;
+  MOUSEPAD_SETTING_SET_ENUM (OPENING_MODE, gtk_combo_box_get_active (combo));
+  self->blocked = FALSE;
+}
+
+
+
+/* update the combo when the setting changes */
+static void
+mousepad_prefs_dialog_opening_mode_setting_changed (MousepadPrefsDialog *self,
+                                                    gchar               *key,
+                                                    GSettings           *settings)
+{
+  GtkComboBox *combo;
+
+  /* don't do anything when the combo box is itself updating the setting */
+  if (self->blocked)
+    return;
+
+  combo = GTK_COMBO_BOX (gtk_builder_get_object (self->builder, WID_OPENING_MODE_COMBO));
+
+  gtk_combo_box_set_active (combo, MOUSEPAD_SETTING_GET_ENUM (OPENING_MODE));
+}
+
+
+
 #define mousepad_builder_get_widget(builder, name) \
   GTK_WIDGET (gtk_builder_get_object (builder, name))
 
@@ -409,18 +436,19 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
   GError *error = NULL;
   GtkWidget *notebook;
   GtkWidget *content_area;
+  GtkWidget *button;
   GtkWidget *check, *widget;
 
   self->builder = gtk_builder_new ();
 
   /* load the gtkbuilder xml that is compiled into the binary, or else die */
-  if (! gtk_builder_add_from_string (self->builder,
-                                     mousepad_prefs_dialog_ui,
-                                     mousepad_prefs_dialog_ui_length,
-                                     &error))
+  if (! gtk_builder_add_from_resource (self->builder,
+                                       "/org/xfce/mousepad/ui/mousepad-prefs-dialog.ui",
+                                       &error))
     {
       g_error ("Failed to load the internal preferences dialog: %s", error->message);
-      g_error_free (error); // not reached
+      /* not reached */
+      g_error_free (error);
     }
 
   /* add the Glade/GtkBuilder notebook into this dialog */
@@ -430,7 +458,9 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
   gtk_widget_show (notebook);
 
   /* add the close button */
-  gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  button = mousepad_util_image_button ("window-close", _("_Close"));
+  gtk_widget_set_can_default (button, TRUE);
+  gtk_dialog_add_action_widget (GTK_DIALOG (self), button, GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (self), GTK_RESPONSE_CLOSE);
 
   /* setup the window properties */
@@ -445,7 +475,8 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
   /* enable/disable font chooser button when the default font checkbox is changed */
   check = mousepad_builder_get_widget (self->builder, WID_USE_DEFAULT_FONT_CHECK);
   widget = mousepad_builder_get_widget (self->builder, WID_FONT_BUTTON);
-  g_object_bind_property (check, "active", widget, "sensitive", G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+  g_object_bind_property (check, "active", widget, "sensitive",
+                          G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 
   /* setup tab mode combo box */
   widget = mousepad_builder_get_widget (self->builder, WID_TAB_MODE_COMBO);
@@ -470,6 +501,10 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
   g_object_bind_property (check, "active", widget, "sensitive", G_BINDING_SYNC_CREATE);
   mousepad_prefs_dialog_toolbar_icon_size_setting_changed (self, NULL, NULL);
 
+  /* setup opening mode combo box */
+  widget = mousepad_builder_get_widget (self->builder, WID_OPENING_MODE_COMBO);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (widget), MOUSEPAD_SETTING_GET_ENUM (OPENING_MODE));
+
   /* bind checkboxes to settings */
 #define BIND_CHECKBOX(setting)                                           \
   MOUSEPAD_SETTING_BIND (setting,                                        \
@@ -571,6 +606,18 @@ mousepad_prefs_dialog_init (MousepadPrefsDialog *self)
                                    G_CALLBACK (mousepad_prefs_dialog_toolbar_icon_size_setting_changed),
                                    self,
                                    G_CONNECT_SWAPPED);
+
+  /* update opening mode when changed */
+  g_signal_connect_swapped (gtk_builder_get_object (self->builder, WID_OPENING_MODE_COMBO),
+                            "changed",
+                            G_CALLBACK (mousepad_prefs_dialog_opening_mode_changed),
+                            self);
+
+  /* update opening mode combo when setting changes */
+  MOUSEPAD_SETTING_CONNECT_OBJECT (OPENING_MODE,
+                                   G_CALLBACK (mousepad_prefs_dialog_opening_mode_setting_changed),
+                                   self,
+                                   G_CONNECT_SWAPPED);
 }
 
 
